@@ -1,7 +1,7 @@
 from functools import wraps
 
 from urecord import Record
-from urlobject import URLObject, encode_component
+from urlobject import URLObject
 
 from conneg.accept import make_accept_header
 from conneg.default import DEFAULT_REGISTER
@@ -55,7 +55,7 @@ class API(Record('url', 'http')):
 
     def __new__(cls, url, http=urllib3_http_backend):
         if not isinstance(url, URLObject):
-            url = URLObject.parse(url)
+            url = URLObject(url)
         return super(API, cls).__new__(cls, url, http)
 
     def __repr__(self):
@@ -83,47 +83,87 @@ class API(Record('url', 'http')):
         return unicode(self) + other
 
     @replaces_url
-    def __and__(self, params):
+    def add_query_params(self, *args, **kwargs):
 
         """
         Add parameters to this URL.
 
-        Accepts pairs or dictionaries:
+        Accepts dictionaries, lists of pairs, and keyword arguments:
 
-            >>> API(u'http://example.com/') & ('a', 'b')
-            <intessa.API(u'http://example.com/?a=b')>
-            >>> API(u'http://example.com/') & {'a': 'b'}
+            >>> API(u'http://example.com/').add_query_params({'a': 'b'})
             <intessa.API(u'http://example.com/?a=b')>
+            >>> API(u'http://example.com/').add_query_params(key='value')
+            <intessa.API(u'http://example.com/?key=value')>
 
         Multiple additions of the same parameter will keep all of them, in the
-        order they were specified:
+        order they were given:
 
-            >>> API(u'http://example.com/') & ('a', 'b') & ('a', 'c')
+            >>> API(u'http://example.com/').add_query_params([
+            ...     ('a', 'b'), ('a', 'c')])
             <intessa.API(u'http://example.com/?a=b&a=c')>
         """
 
-        return self.url & params
+        return self.url.add_query_params(*args, **kwargs)
 
     @replaces_url
-    def __or__(self, params):
+    def add_query_param(self, key, value=None):
+
+        """
+        Add a single parameter to this URL.
+
+        Accepts a key and a value:
+
+            >>> API(u'http://example.com/').add_query_param('a', 'b')
+            <intessa.API(u'http://example.com/?a=b')>
+            >>> API(u'http://example.com/').add_query_param('a', '')
+            <intessa.API(u'http://example.com/?a=')>
+            >>> API(u'http://example.com/').add_query_param('a')
+            <intessa.API(u'http://example.com/?a')>
+        """
+
+        return self.url.add_query_param(key, value)
+
+    @replaces_url
+    def set_query_params(self, *args, **kwargs):
 
         """
         Add parameters to this URL, replacing existing parameters.
 
-        Accepts pairs or dictionaries:
+        Accepts dictionaries, lists of pairs, and keyword arguments:
 
-            >>> API(u'http://example.com/') | ('a', 'b')
-            <intessa.API(u'http://example.com/?a=b')>
-            >>> API(u'http://example.com/') | {'a': 'b'}
+            >>> API(u'http://example.com/?a=foo').set_query_params({'a': 'b'})
             <intessa.API(u'http://example.com/?a=b')>
+            >>> API(u'http://example.com/?key=oldvalue').set_query_params( \\
+            ...     key='newvalue')
+            <intessa.API(u'http://example.com/?key=newvalue')>
 
-        Multiple additions of the same parameter will replace old values:
+        Multiple additions of the same parameter will replace existing values:
 
-            >>> API(u'http://example.com/') | ('a', 'b') | ('a', 'c')
+            >>> API(u'http://example.com/') \\
+            ...     .set_query_params({'a': 'b'}) \\
+            ...     .set_query_params({'a': 'c'})
             <intessa.API(u'http://example.com/?a=c')>
         """
 
-        return self.url | params
+        return self.url.set_query_params(*args, **kwargs)
+
+    @replaces_url
+    def set_query_param(self, key, value=None):
+
+        """
+        Add a single parameter to this URL, replacing it if it already exists.
+
+        Accepts a key and a value:
+
+            >>> API(u'http://example.com/?a=foo').set_query_param('a', 'b')
+            <intessa.API(u'http://example.com/?a=b')>
+            >>> API(u'http://example.com/?a=foo').set_query_param('a', '')
+            <intessa.API(u'http://example.com/?a=')>
+            >>> API(u'http://example.com/?a=foo').set_query_param('a')
+            <intessa.API(u'http://example.com/?a')>
+        """
+
+        return self.url.set_query_param(key, value)
 
     @replaces_url
     def __getattr__(self, attr):
@@ -208,14 +248,12 @@ class API(Record('url', 'http')):
             return self._with_file_ext(item[1:])
 
         # Standard case of adding a path component.
-        return self.url / item
+        return self.url.add_path(item)
 
     def _with_url(self, url):
         """Replace the URL on this API with another."""
 
-        if not isinstance(url, URLObject):
-            url = URLObject.parse(url)
-        return self._replace(url=url)
+        return self._replace(url=URLObject(url))
 
     @replaces_url
     def _with_file_ext(self, file_ext):
@@ -247,18 +285,18 @@ class API(Record('url', 'http')):
 
         """
 
-        if self.url.endswith('/'):
+        if not self.url.is_leaf:
             raise ValueError("Cannot add a file extension to directories.")
 
         # 'https://example.com/foo/bar/baz.xyz' => 'baz.xyz'
-        leaf_node = self.url.path_list()[-1]
+        leaf_node = self.url.path.segments[-1]
         # 'baz.xyz' => 'baz'
         base = leaf_node.rsplit('.', 1)[0]
 
         if not file_ext.startswith('.'):
             file_ext = '.' + file_ext
 
-        return self.url.parent() / (base + file_ext)
+        return self.url.parent.add_path_segment(base + file_ext)
 
     def __call__(self, *args, **kwargs):
 
@@ -313,6 +351,9 @@ class API(Record('url', 'http')):
         :class:`intessa.Error` will be *raised*.
         """
 
+        if 'params' in kwargs:
+            self = self.add_query_params(kwargs.pop('params'))
+
         method = kwargs.pop('method', 'GET')
         headers = CaselessDictionary(kwargs.pop('headers', {}))
         codec_register = kwargs.pop('codec_register', DEFAULT_REGISTER)
