@@ -4,11 +4,10 @@ from text import TextCodec
 from json import JSONCodec
 from forms import SimpleFormCodec
 from multipart import MultipartFormCodec
-from xml import XMLCodec
 
 
 __all__ = ['DEFAULT_REGISTER', 'TextCodec', 'JSONCodec', 'SimpleFormCodec',
-           'MultipartFormCodec', 'XMLCodec']
+           'MultipartFormCodec']
 
 
 DEFAULT_REGISTER = CodecRegister()
@@ -27,5 +26,11 @@ DEFAULT_REGISTER.alias('form', 'application/x-www-form-urlencoded')
 DEFAULT_REGISTER['multipart/form-data'] = MultipartFormCodec
 DEFAULT_REGISTER.alias('multipart', 'multipart/form-data')
 
-DEFAULT_REGISTER['application/xml'] = XMLCodec
-DEFAULT_REGISTER.alias('xml', 'application/xml')
+try:
+    from xml import XMLCodec
+except ImportError:
+    pass
+else:
+    __all__.append('XMLCodec')
+    DEFAULT_REGISTER['application/xml'] = XMLCodec
+    DEFAULT_REGISTER.alias('xml', 'application/xml')
